/*
        cstring.h

        V2.00 - 180297  Kimmo Tervinen
        -----   ------  ----------------
        V1.01   180297  Started to create 2nd release of string class.
                        Reason for this is failures to link GNU String
                        with libnix.
        V1.10   190297  Tested shortly. Seem to work.
        V1.11   190297  Added lastchar().
        V1.12   190297  Added +=(const char) & =(const char)
        V1.13   190297  Added ==
*/
#ifndef ACL_CSTRING
#define ACL_CSTRING

#include <exec/types.h>

class cstring {
  char *data;
  int size;
protected:
  void cstrcopy(const char*,char*,int);
  int cstrlength(const char*) const;

  void Alloc(int Size) {
    Delete();
    if(Size>0) {
      data=new char[Size];
      if(data) {
        size=Size;
        data[0]=0;
      }
    }
  }
  void ReAlloc(int Size) {
    if(Size>0) {
      char *t=new char[Size];
      if(t) {
        cstrcopy(data,t,Size);
        Delete();
        data=t; size=Size;
      }
    }
  }
  void Delete() { if(data) delete [] data; data=NULL; size=0; }
public:
  cstring(const char *text=NULL) : data(NULL), size(0) { *this=text; }
  ~cstring() { Delete(); }

  clear() { if(data) data[0]=0; }

  const char *chars() const { return data; }
  int length() const { return cstrlength(data); }
  char lastchar() const;

  operator const char*() const { return data; }
  operator UBYTE*() const { return (UBYTE*)data; } // for AmigaOS
  operator int() const { return length(); }

  cstring &operator=(const char *text) {
    int l=1;
    if(text) l+=cstrlength(text);
    Alloc(l);
    cstrcopy(text,data,size);
    return *this;
  }
  cstring &operator=(const cstring &s) { return *this=s.data; }
  cstring &operator=(const UBYTE *txt) { return *this=(const char*)txt; }
  cstring &operator=(const char c) {
    char cc[2];
    cc[0]=c; cc[1]=0;
    return *this=cc;
  }

  cstring &operator+=(const char *text) {
    if(text) {
      int l=length();
      ReAlloc(l+cstrlength(text)+1);
      cstrcopy(text,&data[l],size-l);
    }
    return *this;
  }
  cstring &operator+=(const cstring &s) { return *this+=s.data; }
  cstring &operator+=(const UBYTE *txt) { return *this+=(const char*)txt; }
  cstring &operator+=(const char c) {
    char cc[2];
    cc[0]=c; cc[1]=0;
    return *this+=cc;
  }

  int operator==(const char *text) const;
  int operator==(const cstring &s) const { return *this==s.data; }
  int operator==(const UBYTE *txt) const { return *this==(const char*)txt; }
  int operator==(const char c) {
    if(length()!=1) return FALSE;
    return data[0]==c;
  }
};

#endif
